/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.handler.axis;

import org.apache.ws.resource.impl.ResourceContextImpl;
import javax.xml.rpc.handler.soap.SOAPMessageContext;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * Axis version of a ResourceContext.
 *
 * @author Sal Campana
 * @author Ian Springer
 */
public class AxisResourceContext
   extends ResourceContextImpl
{
   private static final String PROP_AXIS_TRANS_URL = "transport.url";

   /**
    * Creates a new {@link AxisResourceContext} object.
    *
    * @param msgContext SOAPMessageContext
    */
   public AxisResourceContext( SOAPMessageContext msgContext )
   throws Exception
   {
      super( msgContext );

      //importServiceOptions( msgContext );   removed to alleviate Axis dependency
   }

   /**
    * Returns the Service url from the Axis MessageContext {@link org.apache.axis.MessageContext#getProperty(String)}
    * passing it {@link org.apache.axis.MessageContext#TRANS_URL}
    *
    * @param msgContext Jax-RPC MessageContext
    *
    * @return The service URL
    */
   protected URL getServiceURL( SOAPMessageContext msgContext )
   {
      if ( msgContext == null )
      {
         throw new IllegalArgumentException( "Null parameter: msgContext" );
      }

      try
      {
         return new URL( (String) msgContext.getProperty( PROP_AXIS_TRANS_URL ) );
      }
      catch ( MalformedURLException murle )
      {
         throw new RuntimeException( "Value of Axis " + PROP_AXIS_TRANS_URL
                                     + " MessageContext property is not a valid URL.", murle );
      }
   }

   /**   Removed because we don't think we use it and it is keeping Axis dependency
      private void importServiceOptions( MessageContext msgContext )
      {
         org.apache.axis.MessageContext axisMsgContext = (org.apache.axis.MessageContext) msgContext;
         SOAPService                    service  = axisMsgContext.getService(  );
         Hashtable                      options  = service.getOptions(  );
         Iterator                       iterator = options.keySet(  ).iterator(  );
   
           while ( iterator.hasNext(  ) )
           {
              String key = (String) iterator.next(  );
              setProperty( key,
                           service.getOption( key ) );
           }
        }
    **/
}