/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.handler.websphere;

import org.apache.ws.resource.impl.ResourceContextImpl;
import javax.xml.rpc.handler.soap.SOAPMessageContext;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * Websphere version of a ResourceContext.
 *
 * @author Sal Campana
 */
public class WASResourceContext
   extends ResourceContextImpl
{
   private static final String PROP_WAS_INBOUND_URL = "inbound.url";

   /**
    * Creates a new {@link ResourceContextImpl} object.
    *
    * @param soapMsgContext SOAP message context corresponding to a particular request
    */
   public WASResourceContext( SOAPMessageContext soapMsgContext )
   throws Exception
   {
      super( soapMsgContext );
   }

   /**
    * Returns the Service URL from the WAS MessageContext properties
    *
    * @param msgContext the context for the current SOAP request; must not be null
    * @return DOCUMENT_ME
    */
   protected URL getServiceURL( SOAPMessageContext msgContext )
   {
      if ( msgContext == null )
      {
         throw new IllegalArgumentException( "The MessageContext was Null." );
      }

      try
      {
         return new URL( (String) msgContext.getProperty( PROP_WAS_INBOUND_URL ) );
      }
      catch ( MalformedURLException murle )
      {
         throw new RuntimeException( "Value of Websphere " + PROP_WAS_INBOUND_URL
                                     + " MessageContext property is not a valid URL.", murle );
      }
   }
}