/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.properties;

import org.apache.ws.resource.properties.impl.CallbackFailedException;
import javax.xml.namespace.QName;

/**
 * A callback that should be registered on any mutable properties (i.e.
 * properties that support the WSRP SetResourceProperties operation).
 * Register the callback by calling
 * {@link ResourceProperty#setCallback(ResourcePropertyCallback)} on the
 * property.
 */
public interface SetResourcePropertyCallback
   extends ResourcePropertyCallback
{
   /**
    * Implement this method for handling "delete" on the backend.
    *
    * @param propQName
    * @throws CallbackFailedException An exception if the delete fails on the backend
    * and you want the server to return a fault.
    */
   void deleteProperty( QName propQName )
   throws CallbackFailedException;

   /**
    * Implement this method for handling "insert" on the backend.
    *
    * @param propElems
    * @throws CallbackFailedException  An exception if the insert fails on the backend
    * and you want the server to return a fault.
    */
   void insertProperty( Object[] propElems )
   throws CallbackFailedException;

   /**
    * Implement this method for handling "update" on the backend.
    *
    * @param propElems
    * @throws CallbackFailedException  An exception if the update fails on the backend
    * and you want the server to return a fault.
    */
   void updateProperty( Object[] propElems )
   throws CallbackFailedException;
}