/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.util.jndi;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import javax.naming.Context;
import javax.naming.Name;
import javax.naming.NamingException;
import java.util.Hashtable;

/**
 * An extension of the Commons Naming {@link org.apache.naming.factory.BeanFactory}
 * that overrides
 * {@link org.apache.naming.factory.BeanFactory#getObjectInstance(Object, javax.naming.Name, javax.naming.Context, java.util.Hashtable)}
 * to add support for calling init() on any bean that implements the {@link Initializable} interface.
 *
 * @author Globus
 */
public class BasicBeanFactory
   extends org.apache.naming.factory.BeanFactory
{
   private static final Log LOG = LogFactory.getLog( BasicBeanFactory.class );

   /**
    * Create a new Bean instance. If the created bean implements
    * <code>Initializable</code> interface, the <code>initialize()</code>
    * function will be called after the bean is created and all properties
    * are set.
    */
   public Object getObjectInstance( Object    obj,
                                    Name      name,
                                    Context   nameCtx,
                                    Hashtable environment )
   throws NamingException
   {
      LOG.debug( "Creating instance of " + obj.getClass(  ).getName(  ) + " bean "
                 + " and adding to JNDI registry with name " + name + " ..." );
      Object bean = super.getObjectInstance( obj, name, nameCtx, environment );
      if ( bean instanceof Initializable )
      {
         try
         {
            LOG.debug( "Calling init() on the bean: " + bean.getClass(  ).getName(  ) );
            ( (Initializable) bean ).init(  );
         }
         catch ( Exception e )
         {
            NamingException ex = new NamingException( "beanInitFailed" );
            ex.setRootCause( e );
            throw ex;
         }
      }

      return bean;
   }
}