/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.properties;

import org.apache.ws.resource.NamespaceVersionHolder;
import org.apache.ws.resource.PropertiesResource;
import org.apache.ws.resource.Resource;
import org.apache.ws.resource.impl.AbstractResourceHome;
import org.apache.ws.resource.properties.impl.XmlBeansResourcePropertySet;
import org.apache.ws.resource.properties.v2004_06.impl.NamespaceVersionHolderImpl;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlOptions;
import javax.xml.namespace.QName;
import java.io.InputStream;
import java.io.Serializable;
import java.util.Map;

/**
 * @author Sal Campana
 */
public class SushiHome
   extends AbstractResourceHome
   implements Serializable
{
   /**
    * The service endpoint name as registered with the SOAP Platform.  This is useful for building EPR's. *
    */
   public static final QName SERVICE_NAME =
      javax.xml.namespace.QName.valueOf( "{http://ws.apache.org/resource/properties/test/sushi}sushi" );

   /**
    * The management PortType associated with this resource. This is useful for building EPR's.*
    */
   public static final QName PORT_TYPE =
      javax.xml.namespace.QName.valueOf( "{http://ws.apache.org/resource/properties/test/sushi}sushi" );

   /**
    * The WSDL Port name associated with the resource. This is useful for building EPR's. *
    */
   public static final String PORT_NAME = "sushi";

   /**
    * The name of the resource key for this resource. *
    */
   public static final QName RESOURCE_ID_REF_PARAM_NAME =
      javax.xml.namespace.QName.valueOf( "{http://ws.apache.org/resource/properties/test/sushi}ResourceID" );

   /**
    * DOCUMENT_ME
    */
   protected static String                     RESOURCE_PROPS_RESOURCE_PATH =
      "org/apache/ws/resource/properties/SushiProperties.xml";
   private static final NamespaceVersionHolder SPEC_NAMESPACE_SET = new NamespaceVersionHolderImpl(  );

   /**
    * Map containing all SushiResource instances - this map <em>must</em> be static for compatibility with certain JNDI
    * providers.
    */
   private static Map s_resources;
   private boolean    m_isOpenContent;

   /**
    * Creates a new {@link SushiHome} object.
    *
    * @param isOpenContent DOCUMENT_ME
    *
    * @throws Exception DOCUMENT_ME
    */
   public SushiHome( boolean isOpenContent )
   throws Exception
   {
      m_isOpenContent = isOpenContent;
      setResourceClassName( SushiResource.class.getName(  ) );
      setResourceIdentifierReferenceParameterName( RESOURCE_ID_REF_PARAM_NAME.toString(  ) );
      resetResourceMap(  );
      init(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public QName getPortType(  )
   {
      return PORT_TYPE;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public QName getResourceIdentifierReferenceParameterQName(  )
   {
      return RESOURCE_ID_REF_PARAM_NAME;
   }

   /**
    * DOCUMENT_ME
    *
    * @param propsDoc DOCUMENT_ME
    */
   public void setResourcePropsDocPath( String propsDoc )
   {
      SushiHome.RESOURCE_PROPS_RESOURCE_PATH = propsDoc;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public QName getServiceName(  )
   {
      return SERVICE_NAME;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getServicePortName(  )
   {
      return PORT_NAME;
   }

   /**
    * DOCUMENT_ME
    *
    * @throws Exception DOCUMENT_ME
    */
   public void init(  )
   throws Exception
   {
      super.init(  );
      InputStream in        = getClass(  ).getClassLoader(  ).getResourceAsStream( RESOURCE_PROPS_RESOURCE_PATH );
      XmlOptions  parseOpts = new XmlOptions(  ).setLoadStripWhitespace(  );
      if ( !m_isOpenContent )
      {
         parseOpts.setLoadReplaceDocumentElement( new QName( AbstractResourcePropertiesTestCase.NSURI_SUSHI,
                                                             "ClosedSushiProperties", "sushi" ) );
      }

      XmlObject propsDocXBean = XmlObject.Factory.parse( in, parseOpts );
      in.close(  );
      ResourcePropertySet propSet  = new XmlBeansResourcePropertySet( propsDocXBean );
      Resource            resource = createInstance( SushiResourceContext.ID );
      ( (PropertiesResource) resource ).setResourcePropertySet( propSet );
      add( resource );
   }

   /**
    * Resets (clears) the resource map.
    */
   public final synchronized void resetResourceMap(  )
   {
      if ( s_resources != null )
      {
         s_resources.clear(  );
      }
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public org.apache.ws.resource.NamespaceVersionHolder getNamespaceSet(  )
   {
      return SPEC_NAMESPACE_SET;
   }

   /**
    * Returns a map of all SushiResource instances. Used by the {@link AbstractResourceHome} superclass.
    */
   protected final synchronized Map getResourceMap(  )
   {
      if ( s_resources == null )
      {
         s_resources = AbstractResourceHome.createResourceMap( m_resourceIsPersistent );
      }

      return s_resources;
   }
}